
local YssPersistantWatch = CreateFrame("Frame")
YssPersistantWatch:SetScript("OnEvent", function(self, event, ...) if self[event] then return self[event](self, event, ...) end end)
YssPersistantWatch:RegisterEvent("ADDON_LOADED")
YssPersistantWatch:RegisterEvent("PLAYER_LOGOUT")

function YssPersistantWatch:ADDON_LOADED(event, addon)
	if addon == "YssPersistantWatch" then
		if IsLoggedIn() then self:PLAYER_LOGIN() else self:RegisterEvent("PLAYER_LOGIN") end
		self:UnregisterEvent("ADDON_LOADED")
		self.ADDON_LOADED = nil
	end
end

function YssPersistantWatch:PLAYER_LOGIN(event, addon)
	local defaults = {
		["char"] = {
			['Achievement'] = nil,
			['Quests'] = {},
			['CollapsedHeaders'] = {},
		},
	}
	YssPersistantWatch_DB = YssPersistantWatch_DB or {
			['Achievement'] = nil, -- just for reference
			['Quests'] = {},
			['CollapsedHeaders'] = {},
		}
	self.db = YssPersistantWatch_DB
	if self.db.Achievement then
		AddTrackedAchievement(self.db.Achievement)
	end
	ExpandQuestHeader(0)
	self:RegisterEvent('QUEST_LOG_UPDATE')
	self:UnregisterEvent("PLAYER_LOGIN")
	self.PLAYER_LOGIN = nil
end

function YssPersistantWatch:QUEST_LOG_UPDATE()
	local notExpanded = false
	local numEntries = GetNumQuestLogEntries()
	for i=1,numEntries do
		local _, _, _, _, isHeader, isCollapsed = GetQuestLogTitle(i)
		if isCollapsed then
			notExpanded = true
		end
	end
	if notExpanded then
		return
	else
		for i=numEntries,1,-1 do
			local questTitle, _, _, _, isHeader, _, _, _, questID = GetQuestLogTitle(i)
			if isHeader and self.db.CollapsedHeaders[questTitle] then
				CollapseQuestHeader(i)
			elseif not isHeader and self.db.Quests[questID] then
				AddQuestWatch(i)
			end
		end
		QuestLog_Update()
		WatchFrame_Update()
		self:UnregisterEvent("QUEST_LOG_UPDATE")
		self.QUEST_LOG_UPDATE = nil
	end
end

function YssPersistantWatch:PLAYER_LOGOUT(event)
	self.db.Achievement = GetTrackedAchievements()
	local numQuests = GetNumQuestWatches()
	wipe(self.db.Quests)
	for i=1, numQuests do
		local _, _, _, _, _, _, _, _, questID = GetQuestLogTitle(GetQuestIndexForWatch(i))
		self.db.Quests[questID] = true
	end
	wipe(self.db.CollapsedHeaders)
	local numEntries = GetNumQuestLogEntries()
	for i=1,numEntries do
		local questTitle, _, _, _, isHeader, isCollapsed = GetQuestLogTitle(i)
		if isHeader and isCollapsed then
			self.db.CollapsedHeaders[questTitle] = true
		end
	end
end
